from django import template
from django.template.defaultfilters import stringfilter
from currencies.models import Currency
from currencies.utils import calculate_price

register = template.Library()


@register.filter(name='currency')
@stringfilter
def set_currency(value, arg):
	return calculate_price(value, arg)


class ChangeCurrencyNode(template.Node):

	def __init__(self, price, currency):
		self.price = template.Variable(price)
		self.currency = template.Variable(currency)

	def render(self, context):
		try:
			return calculate_price(self.price.resolve(context),
				self.currency.resolve(context))
		except template.VariableDoesNotExist:
			return ''


@register.tag(name='change_currency')
def change_currency(parser, token):
	try:
		tag_name, current_price, new_currency = token.split_contents()
	except ValueError:
		raise template.TemplateSyntaxError, \
			'%r tag requires exactly two arguments' % token.contents.split()[0]
	return ChangeCurrencyNode(current_price, new_currency)



from decimal import *
from django.test import TestCase
from django.conf import settings
from django import template
from currencies.models import Currency
from currencies.utils import calculate_price



class UtilsTest(TestCase):
	fixtures = ['test_data.json']
	use_transaction = False
	
	def test_calculate_price_success(self):
		res = calculate_price('10', 'USD')
		self.assertEqual(res, Decimal('15.00'))

	def test_calculate_price_fail(self):
		res = calculate_price('10', 'GBP')
		self.assertEqual(res, Decimal('0.00'))
	
	def test_calculate_price_currency_doesnotexist(self):
		settings.DEBUG = True
		self.assertRaises(Currency.DoesNotExist, calculate_price, '10', 'GBP')
		settings.DEBUG = False


class TemplateTagTest(TestCase):
	fixtures = ['test_data.json']
	use_transaction = False

	def setUp(self):
		self.template = '{% load currency %}'

	def test_currency_filter(self):
		t = template.Template(self.template +
			'{{ 10|currency:"USD" }}'
		)
		c = template.Context()
		s = t.render(c)
		self.assertEqual(s, u'15.00')

	def test_change_currency_tag_success(self):
		t = template.Template(self.template +
			'{% change_currency 10 "USD" %}'
		)
		c = template.Context()
		s = t.render(c)
		self.assertEqual(s, u'15.00')

	def test_change_currency_tag_fail(self):
		t = template.Template(self.template +
			'{% change_currency 10 "GPB" %}'
		)
		c = template.Context()
		s = t.render(c)
		self.assertEqual(s, u'0.00')



# -*- coding: utf-8 -*-
import os

DATABASE_ENGINE = 'sqlite3'
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.gzip.GZipMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
)

ROOT_URLCONF = 'holabyolga.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'currencies',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.request',
)



from django.db import models
from django.utils.translation import gettext_lazy as _


class Currency(models.Model):
	code = models.CharField(_('code'), max_length=3)
	name = models.CharField(_('name'), max_length=25)
	symbol = models.CharField(_('symbol'), max_length=1)
	factor = models.DecimalField(_('factor'), max_digits=10, decimal_places=4,
		help_text=_('Specifies the difference of the currency to default one.'))
	is_active = models.BooleanField(_('active'), default=True,
		help_text=_('The currency will be available.'))
	is_default = models.BooleanField(_('default'), default=False,
		help_text=_('Make this the default currency.'))

	class Meta:
		verbose_name = _('currency')
		verbose_name_plural = _('currencies')

	def __unicode__(self):
		return self.code

	def save(self, **kwargs):
		if self.is_default:
			try:
				default_currency = Currency.objects.get(is_default=True)
			except DoesNotExist:
				pass
			else:
				default_currency.is_default = False
				default_currency.save()
		super(Currency, self).save(**kwargs)



from decimal import *
from django.conf import settings
from currencies.models import Currency


def calculate_price(price, currency):
	try:
		factor = Currency.objects.get(code__exact=currency).factor
	except Currency.DoesNotExist:
		if settings.DEBUG:
			raise Currency.DoesNotExist
		else:
			factor = Decimal('0.0')
	new_price = Decimal(price) * factor
	return new_price.quantize(Decimal('.01'), rounding=ROUND_UP)



from django.http import HttpResponseRedirect
from currencies.models import Currency


def set_currency(request):
	if request.method == 'POST':
		currency_code = request.POST.get('currency', None)
		next = request.POST.get('next', None)
	else:
		currency_code = request.GET.get('currency', None)
		next = request.GET.get('next', None)
	if not next:
		next = request.META.get('HTTP_REFERER', None)
	if not next:
		next = '/'
	response = HttpResponseRedirect(next)
	if currency_code:
		if hasattr(request, 'session'):
			request.session['currency'] = \
				Currency.objects.get(code__exact=currency_code)
		else:
			response.set_cookie('currency', currency_code)
	return response



from django.contrib import admin
from currencies.models import Currency


class CurrencyAdmin(admin.ModelAdmin):
	list_display = ('is_default', 'code', 'name', 'symbol', 'factor')
	list_display_links = ('name',)

admin.site.register(Currency, CurrencyAdmin)



#!/usr/bin/env python
from distutils.core import setup
import os
from currencies import __version__


# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
	os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk('currencies'):
	# Ignore dirnames that start with '.'
	for i, dirname in enumerate(dirnames):
		if dirname.startswith('.'): del dirnames[i]
	if '__init__.py' in filenames:
		pkg = dirpath.replace(os.path.sep, '.')
		if os.path.altsep:
			pkg = pkg.replace(os.path.altsep, '.')
		packages.append(pkg)
	elif filenames:
		prefix = dirpath[11:] # Strip "currencies/" or "currencies\"
		for f in filenames:
			data_files.append(os.path.join(prefix, f))

setup(name='django-currencies',
	version=__version__,
	description='Adds support for multiple currencies as a Django application.',
	long_description=open('README.txt').read(),
	author='Panos Laganakos',
	author_email='panos.laganakos@gmail.com',
	url='http://code.google.com/p/django-currencies/',
	package_dir={'currencies': 'currencies'},
	packages=packages,
	package_data={'currencies': data_files},
	classifiers=['Development Status :: 4 - Beta',
		'Environment :: Web Environment',
		'Framework :: Django',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: BSD License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Topic :: Utilities'],
	)



from currencies.models import Currency


def currencies(request):
	currencies = Currency.objects.all()

	if not request.session.get('currency'):
		#request.session['currency'] = Currency.objects.get(code__exact='EUR')
		request.session['currency'] = Currency.objects.get(is_default__exact=True)

	return {
		'CURRENCIES': currencies,
		'currency': request.session['currency'], # DEPRECATED
		'CURRENCY': request.session['currency']
	}
